using System.Collections;
using UnityEngine;


[System.Serializable]
public struct EntityHealthProperties
{
    public float maxHealth;
    public float currentHealth;

   
    public void Reset()
    {
        currentHealth = maxHealth;
    }
}


public class Spider : MonoBehaviour, IDamagableEntity
{
    [SerializeField] private EntityHealthProperties healthProperties;



    public float minSpeed;

    public GameObject bloodSplash;
   public SpriteRenderer spriteRenderer;


    public DialogueStructure hurtDialogue;
    public Rigidbody2D riggy;
    public TextMesh healthVisual;

    public TextMesh thought;

    public GameObject drop;

    private float framesMiss = 0;

    protected PlayerController player;


    private float trueSpeed;
    
    private void Awake()
    {
    //  spriteRenderer = GetComponent<SpriteRenderer>();
       
        player = FindObjectOfType<PlayerController>();

        riggy = GetComponent<Rigidbody2D>();
        trueSpeed = Random.Range(minSpeed, minSpeed + minSpeed / 2);
        int i0 = LayerMask.NameToLayer("drop");
        int i2 = LayerMask.NameToLayer("enemy");

        Physics2D.IgnoreLayerCollision(i0, i2, true);

    }


    protected virtual void OnValidate()
    {
        if(healthVisual != null)
        {
            healthProperties.Reset();
            healthVisual.text = healthProperties.currentHealth.ToString();
        }
       
    }

  
  

    protected virtual void OnDeath()
    {
        Instantiate(bloodSplash, transform.position, Quaternion.identity);
        Destroy(gameObject);
        if (drop != null) Instantiate(drop, transform.position, Quaternion.identity);
    }

    // receieve
    public void TakeDamage(float amount)
    {
        if (framesMiss > 0) return;

        startAggro = true;
        healthProperties.currentHealth -= amount;
        if (canSpeak)
        {
            thought.text = hurtDialogue.GetRandomDialogue();
            StartCoroutine(DialogueTimer());
            
        }
        if(healthProperties.currentHealth <= 0)
        {
            OnDeath();
            thought.text = "why..";
            return;
        }
        healthVisual.text = healthProperties.currentHealth.ToString();
        framesMiss += 0.2f;
        StartCoroutine(HitEffect());
        StartCoroutine(Invun());

    }


    private void OnTriggerEnter2D(Collider2D collision)
    {
       
        PlayerController p = collision.GetComponent<PlayerController>();
        if(p != null)
        {
            startAggro = true;
            p.TakeDamage(5);
        }
    }


    private bool startAggro;

    private float enemyChecker;
    protected virtual void  Update()
    {
        enemyChecker += Time.deltaTime;

        if (enemyChecker >= 2)
        {
            enemyChecker = 0;
            var a = Physics2D.OverlapCircleAll(transform.position, 20);
          
            foreach (var col in a)
            {
                
                PlayerController p = col.GetComponent<PlayerController>();
                if(p != null)
                {
                    startAggro = true;
                }

            }
        }


        if (startAggro)
        {
         transform.position = Vector2.MoveTowards(transform.position, player.transform.position, Time.deltaTime * trueSpeed);

        }
        
     
    }

    IEnumerator HitEffect()
    {
        if (spriteRenderer == null) yield break;
        spriteRenderer.color = Color.red;
        yield return new WaitForSeconds(0.3f);
        spriteRenderer.color = Color.black;
       
    }


    private bool canSpeak = true;
    IEnumerator DialogueTimer()
    {
        thought.gameObject.SetActive(true);
        canSpeak = false;
        float timer = 0;
        while (timer < 4)
        {
            timer += Time.deltaTime;
            yield return null;
            
        }
        canSpeak = true;
        thought.gameObject.SetActive(false);

    }
    IEnumerator Invun()
    {
        while (framesMiss > 0)
        {
            framesMiss -= Time.deltaTime;
            yield return null;


        }
        framesMiss = 0;
    }


    private bool canKnock = true;

    private IEnumerator StartKnockTimer()
    {
        float timer = 1.2f;
        while (timer > 0)
        {
            timer -= Time.deltaTime;
            yield return null;
        }
        canKnock = true;
    }
    public void PushBack(Vector2 direction)
    {
        if (!canKnock) return;


        canKnock = false;
        StartCoroutine(StartKnockTimer());
        riggy.AddForce(direction, ForceMode2D.Impulse);
    }
}
