using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Events;

public static class WorlDialogueEventSystem
{
    public static event Action<int> onPlayerPassage;
    public static event Action<int> onForceDelete;
    public static void CallOnPlayerPassage(int id)
    {
        if (onPlayerPassage != null) onPlayerPassage.Invoke(id);
    }

    public static void CallForceDelete(int id)
    {
        if (onPlayerPassage != null) onForceDelete.Invoke(id);
    }

}

public enum DialogueState
{
    Waiting,
    Appearing,
    Dissapaering
}

public class WorldDialogueController : MonoBehaviour
{

    [Header("Spaguetti boy")]
    public UnityEvent OnDialogueDisplay;
    public UnityEvent OnDialogueEnd;

    public int ID;
    public TextMesh textMesh;
    public float delay;
    public float duration;
    public DialogueState state;

    public bool shouldKill;
 
    float timer;


     float speed => 255 / duration;

    private void Awake()
    {
        SetAlpha(0);
        WorlDialogueEventSystem.onPlayerPassage += TriggerDialogue;
        WorlDialogueEventSystem.onForceDelete += ForceDelete;
    }
    private void OnDestroy()
    {
        WorlDialogueEventSystem.onPlayerPassage -= TriggerDialogue;
        WorlDialogueEventSystem.onForceDelete -= ForceDelete;
    }


    public void SetText(string text, float duration, int delay)
    {
        gameObject.SetActive(true);
        textMesh.text = text;
        this.duration = duration;
        this.delay = delay;
        state = DialogueState.Appearing;
    }

    public void ForceDelete(int id)
    {
        if(id == ID)
          state = DialogueState.Dissapaering;
    }

    private void OnValidate()
    {
        name = $"World Dialogue ID: {ID}";
    }
    private float GetAlpha()
    {
        return textMesh.color.a;
    }
    private void SetAlpha(float amount)
    {
        Color c0 = textMesh.color;
        Color c1 = new Color(c0.r, c0.g, c0.b, amount/100);
        textMesh.color = c1;
    }

    public void TriggerDialogue(int id)
    {
        if (id != ID) return;
        if(state == DialogueState.Waiting)
        {
            
            state = DialogueState.Appearing;
        }
             
    }


    private float delayTimer;

    private bool quick = true;
    private void Update()
    {

        switch (state)
        {
            case DialogueState.Waiting:
                break;
            case DialogueState.Appearing:


                delayTimer += Time.deltaTime;
                if (delayTimer < delay) return;


                if (quick)
                {
                    OnDialogueDisplay?.Invoke();
                    quick = false;
                }

                timer += Time.deltaTime * speed;
                SetAlpha(timer);

                if (GetAlpha() >= 1)
                {

                    timer = 255;
                    state = DialogueState.Dissapaering;
                }
                break;
            case DialogueState.Dissapaering:

                timer -= Time.deltaTime * speed;

                SetAlpha(timer);

                if (GetAlpha() <= 0)
                {
                    OnDialogueEnd?.Invoke();
                    if (shouldKill)
                    {
                        Destroy(gameObject);
                    }
                    else
                    {
                        gameObject.SetActive(false);
                        state = DialogueState.Waiting;
                    }
                 

                }
                
                break;
            default:
                break;
        }
       
    }
}
